@@ -28,12 +28,12 @@ public:
     static constexpr std::size_t max_static_buffer =
         sizeof(beast::detail::temporary_buffer);
 
-    BOOST_STATIC_ASSERT(is_fields<fields>::value);
+    BOOST_CORE_STATIC_ASSERT(is_fields<fields>::value);
 
     // std::allocator is noexcept movable, fields should satisfy
     // these constraints as well.
-    BOOST_STATIC_ASSERT(std::is_nothrow_move_constructible<fields>::value);
-    BOOST_STATIC_ASSERT(std::is_nothrow_move_assignable<fields>::value);
+    BOOST_CORE_STATIC_ASSERT(std::is_nothrow_move_constructible<fields>::value);
+    BOOST_CORE_STATIC_ASSERT(std::is_nothrow_move_assignable<fields>::value);
 
     template<class Allocator>
     using fa_t = basic_fields<Allocator>;
@@ -188,6 +188,7 @@ public:
                 f1.insert("1", "1");
                 basic_fields<pocma_t> f2;
                 f2 = std::move(f1);
+                BEAST_EXPECT(f1.get_allocator()->nmassign == 1);
                 BEAST_EXPECT(f1.begin() == f1.end());
                 BEAST_EXPECT(f2["1"] == "1");
             }
@@ -199,6 +200,7 @@ public:
                 f1.insert("1", "1");
                 basic_fields<pocma_t> f2;
                 f2 = std::move(f1);
+                BEAST_EXPECT(f1.get_allocator()->nmassign == 0);
                 BEAST_EXPECT(f1.begin() == f1.end());
                 BEAST_EXPECT(f2["1"] == "1");
             }
@@ -225,6 +227,7 @@ public:
                 f1.insert("1", "1");
                 basic_fields<pocca_t> f2;
                 f2 = f1;
+                BEAST_EXPECT(f1.get_allocator()->ncpassign == 1);
                 BEAST_EXPECT(f2["1"] == "1");
             }
             {
@@ -235,6 +238,7 @@ public:
                 f1.insert("1", "1");
                 basic_fields<pocca_t> f2;
                 f2 = f1;
+                BEAST_EXPECT(f1.get_allocator()->ncpassign == 0);
                 BEAST_EXPECT(f2["1"] == "1");
             }
         }
@@ -366,24 +370,24 @@ public:
             // group fields
             fields f;
             f.insert(field::age,   "1");
-            f.insert(field::body,  "2");
-            f.insert(field::close, "3");
-            f.insert(field::body,  "4");
+            f.insert(field::cookie,  "2");
+            f.insert(field::from, "3");
+            f.insert(field::cookie,  "4");
             BEAST_EXPECT(std::next(f.begin(), 0)->name() == field::age);
-            BEAST_EXPECT(std::next(f.begin(), 1)->name() == field::body);
-            BEAST_EXPECT(std::next(f.begin(), 2)->name() == field::body);
-            BEAST_EXPECT(std::next(f.begin(), 3)->name() == field::close);
+            BEAST_EXPECT(std::next(f.begin(), 1)->name() == field::cookie);
+            BEAST_EXPECT(std::next(f.begin(), 2)->name() == field::cookie);
+            BEAST_EXPECT(std::next(f.begin(), 3)->name() == field::from);
             BEAST_EXPECT(std::next(f.begin(), 0)->name_string() == "Age");
-            BEAST_EXPECT(std::next(f.begin(), 1)->name_string() == "Body");
-            BEAST_EXPECT(std::next(f.begin(), 2)->name_string() == "Body");
-            BEAST_EXPECT(std::next(f.begin(), 3)->name_string() == "Close");
+            BEAST_EXPECT(std::next(f.begin(), 1)->name_string() == "Cookie");
+            BEAST_EXPECT(std::next(f.begin(), 2)->name_string() == "Cookie");
+            BEAST_EXPECT(std::next(f.begin(), 3)->name_string() == "From");
             BEAST_EXPECT(std::next(f.begin(), 0)->value() == "1");
             BEAST_EXPECT(std::next(f.begin(), 1)->value() == "2");
             BEAST_EXPECT(std::next(f.begin(), 2)->value() == "4");
             BEAST_EXPECT(std::next(f.begin(), 3)->value() == "3");
-            BEAST_EXPECT(f.erase(field::body) == 2);
+            BEAST_EXPECT(f.erase(field::cookie) == 2);
             BEAST_EXPECT(std::next(f.begin(), 0)->name_string() == "Age");
-            BEAST_EXPECT(std::next(f.begin(), 1)->name_string() == "Close");
+            BEAST_EXPECT(std::next(f.begin(), 1)->name_string() == "From");
         }
         {
             // group fields, case insensitive
@@ -445,6 +449,47 @@ public:
             BEAST_EXPECT(std::next(rng.first, 1)->value() == "4");
             BEAST_EXPECT(std::next(rng.first, 2)->value() == "6");
         }
+
+        // max field name and max field value
+        {
+            fields f;
+            error_code ec;
+            auto fit_name  = std::string(fields::max_name_size,      'a');
+            auto big_name  = std::string(fields::max_name_size + 1,  'a');
+            auto fit_value = std::string(fields::max_value_size,     'a');
+            auto big_value = std::string(fields::max_value_size + 1, 'a');
+
+            f.insert(fit_name, fit_value);
+            f.set(fit_name, fit_value);
+
+            f.insert(field::age, big_name, "", ec);
+            BEAST_EXPECT(ec == error::header_field_name_too_large);
+            f.insert(field::age, "", big_value, ec);
+            BEAST_EXPECT(ec == error::header_field_value_too_large);
+
+            BEAST_THROWS(f.insert(field::age, big_value),     boost::system::system_error);
+            BEAST_THROWS(f.insert(field::age, big_name, ""),  boost::system::system_error);
+            BEAST_THROWS(f.insert(field::age, "", big_value), boost::system::system_error);
+            BEAST_THROWS(f.insert(big_name, ""),              boost::system::system_error);
+            BEAST_THROWS(f.insert("", big_value),             boost::system::system_error);
+            BEAST_THROWS(f.set(field::age, big_value),        boost::system::system_error);
+            BEAST_THROWS(f.set(big_name, ""),                 boost::system::system_error);
+            BEAST_THROWS(f.set("", big_value),                boost::system::system_error);
+        }
+
+        {
+            fields f;
+            BEAST_EXPECT(! f.contains("Content-Type"));
+            f.set("Content-Type", "text/html");
+            BEAST_EXPECT(f.contains("Content-Type"));
+            BEAST_EXPECT(f.contains("content-type"));
+            BEAST_EXPECT(! f.contains(field::user_agent));
+
+            f.insert("AA", "a");
+            f.insert("AA", "b");
+            f.insert("AA", "c");
+            BEAST_EXPECT(f.contains("AA"));            
+        }
     }
 
     struct sized_body
@@ -994,33 +1039,33 @@ public:
     void
     testIssue2085()
     {
-        BOOST_STATIC_ASSERT((! set_test<field, int>::value));
-        BOOST_STATIC_ASSERT((! set_test<field, std::nullptr_t>::value));
-        BOOST_STATIC_ASSERT((! set_test<field, double>::value));
-        BOOST_STATIC_ASSERT((! set_test<string_view, int>::value));
-        BOOST_STATIC_ASSERT((! set_test<string_view, std::nullptr_t>::value));
-        BOOST_STATIC_ASSERT((! set_test<string_view, double>::value));
+        BOOST_CORE_STATIC_ASSERT((! set_test<field, int>::value));
+        BOOST_CORE_STATIC_ASSERT((! set_test<field, std::nullptr_t>::value));
+        BOOST_CORE_STATIC_ASSERT((! set_test<field, double>::value));
+        BOOST_CORE_STATIC_ASSERT((! set_test<string_view, int>::value));
+        BOOST_CORE_STATIC_ASSERT((! set_test<string_view, std::nullptr_t>::value));
+        BOOST_CORE_STATIC_ASSERT((! set_test<string_view, double>::value));
 
-        BOOST_STATIC_ASSERT(( set_test<field, const char*>::value));
-        BOOST_STATIC_ASSERT(( set_test<field, string_view>::value));
-        BOOST_STATIC_ASSERT(( set_test<field, const char(&)[10]>::value));
-        BOOST_STATIC_ASSERT(( set_test<string_view, const char*>::value));
-        BOOST_STATIC_ASSERT(( set_test<string_view, string_view>::value));
-        BOOST_STATIC_ASSERT(( set_test<string_view, const char(&)[10]>::value));
+        BOOST_CORE_STATIC_ASSERT(( set_test<field, const char*>::value));
+        BOOST_CORE_STATIC_ASSERT(( set_test<field, string_view>::value));
+        BOOST_CORE_STATIC_ASSERT(( set_test<field, const char(&)[10]>::value));
+        BOOST_CORE_STATIC_ASSERT(( set_test<string_view, const char*>::value));
+        BOOST_CORE_STATIC_ASSERT(( set_test<string_view, string_view>::value));
+        BOOST_CORE_STATIC_ASSERT(( set_test<string_view, const char(&)[10]>::value));
 
-        BOOST_STATIC_ASSERT((! insert_test<field, int>::value));
-        BOOST_STATIC_ASSERT((! insert_test<field, std::nullptr_t>::value));
-        BOOST_STATIC_ASSERT((! insert_test<field, double>::value));
-        BOOST_STATIC_ASSERT((! insert_test<string_view, int>::value));
-        BOOST_STATIC_ASSERT((! insert_test<string_view, std::nullptr_t>::value));
-        BOOST_STATIC_ASSERT((! insert_test<string_view, double>::value));
+        BOOST_CORE_STATIC_ASSERT((! insert_test<field, int>::value));
+        BOOST_CORE_STATIC_ASSERT((! insert_test<field, std::nullptr_t>::value));
+        BOOST_CORE_STATIC_ASSERT((! insert_test<field, double>::value));
+        BOOST_CORE_STATIC_ASSERT((! insert_test<string_view, int>::value));
+        BOOST_CORE_STATIC_ASSERT((! insert_test<string_view, std::nullptr_t>::value));
+        BOOST_CORE_STATIC_ASSERT((! insert_test<string_view, double>::value));
 
-        BOOST_STATIC_ASSERT(( insert_test<field, const char*>::value));
-        BOOST_STATIC_ASSERT(( insert_test<field, string_view>::value));
-        BOOST_STATIC_ASSERT(( insert_test<field, const char(&)[10]>::value));
-        BOOST_STATIC_ASSERT(( insert_test<string_view, const char*>::value));
-        BOOST_STATIC_ASSERT(( insert_test<string_view, string_view>::value));
-        BOOST_STATIC_ASSERT(( insert_test<string_view, const char(&)[10]>::value));
+        BOOST_CORE_STATIC_ASSERT(( insert_test<field, const char*>::value));
+        BOOST_CORE_STATIC_ASSERT(( insert_test<field, string_view>::value));
+        BOOST_CORE_STATIC_ASSERT(( insert_test<field, const char(&)[10]>::value));
+        BOOST_CORE_STATIC_ASSERT(( insert_test<string_view, const char*>::value));
+        BOOST_CORE_STATIC_ASSERT(( insert_test<string_view, string_view>::value));
+        BOOST_CORE_STATIC_ASSERT(( insert_test<string_view, const char(&)[10]>::value));
     }
 
     template<class T>
@@ -1074,12 +1119,12 @@ public:
     testIssue2517()
     {
         using test_fields = basic_fields<throwing_allocator<char>>;
-        BOOST_STATIC_ASSERT(is_fields<test_fields>::value);
+        BOOST_CORE_STATIC_ASSERT(is_fields<test_fields>::value);
 
         // Check if basic_fields respects throw-constructibility and
         // propagate_on_container_move_assignment of the allocator.
-        BOOST_STATIC_ASSERT(std::is_nothrow_move_constructible<test_fields>::value);
-        BOOST_STATIC_ASSERT(!std::is_nothrow_move_assignable<test_fields>::value);
+        BOOST_CORE_STATIC_ASSERT(std::is_nothrow_move_constructible<test_fields>::value);
+        BOOST_CORE_STATIC_ASSERT(!std::is_nothrow_move_assignable<test_fields>::value);
 
         test_fields f1;
         f1.insert("1", "1");
